<?php
/**
 * Created by PhpStorm.
 * User: rafaelpickbrenne
 * Date: 22/05/14
 * Time: 13:15
 */

namespace Folha\Logic;

use Folha\Entities\FpaFormdirf;
use Folha\Entities\FpaDirfmagnet;

include_once "C:/workfolder/v_220/products/projects/hrm/public/bootstrap.php";
//use Folha\Entities\GpeVinculom;
//use Teknisa\Logic\Exception;
//use Folha\Entities\Base\Folha;
//use Folha\Repositori

class Dirf extends \Doctrine2\ORM\EntityRepository {
    //Para a Classe
    const   nomeArquivo = 'dirf.txt';   //Nome do arquivo que será gerado pela dirf
    const   ql_msg      = '<br>';       //Quebra de linha para as mensagens
    const   ql_dslinha  = "\n";         //Quebra de linha para a descrição do registro do vícnulo no magnético
    const   id13Sal     = '-13S';       //IdeNtificação de campo DIRF específico para o 13º salário (campos M13 e VALOR13)
    const   idInforme   = '-INF';      //Identificação de campo que não serão usados no TXT, mas pode-se utilizá-los no magnético/Informe de Rendimentos
    const   queryPartCdReceita = "WHEN ICF.NREVENTOM = 870 AND IEV.IDINCIDEVENTO = '+' THEN '3562|'\nWHEN VM.NRTIPOVINCULOM = 2 THEN '0588|'";    //Segundo o evento força o código de receita é forçado, sobrepondo-se ao código de receita informado no tipo de vínculo. Caso não se use isto preencher com 'WHEN 1 = 0 THEN NULL'
    private $msgErro;           //Mensagem de erro que acumula todos os campos obrigatórios que não possuem valor
    private $queryPart;         //Fragmento de query para realizar a busca na query
    //Formulário
    private $registrosDirf; //Artmazenas os registros da Dirf
    private $dtCompetencia; //Data de competência
    private $nrCenario; //Cenário utilizado (Real, Cenário 1, Cenário 2, etc.)
    private $nrOcorren; //Ocorrência de cálculo (geralmente é o 1). No _motor e _motor_info este parâmetro é ignorado.
    private $nrOrg;         //Nro da organização em questão
    private $nrForm;        //Número do formulário em questão. Os dados pertinentes serão extratídos daqui
    private $nmForm;        //Nome do formulário
    private $idAtivo;       //S para formulário ativo; N para não ativo
    private $idProcessaVinc;    //S para processar os vínculos; N não processa os vínculos
    private $idGeraMagnetic;    //S para gerar meio magnético (tabela)
    private $idExtinsaoEmpr;    //S indica extinsão da empresa; N empresa ativa
    private $idTipoDeclarac;    //S para Retificada; N para Original
    private $idNaturezaDecl;    //Natureza do declarante. Nros de 0 - 4, 8. Ver Manual DIRF.
    private $idOrdem;           //Ordem de classificação dos vínculos
    private $nomeInicial;       //Nome do vínculo inicial da lista
    private $nomeFinal;         //Nome do vínculo final da lista
    private $dataInicial;       //Data inicial (admissão) da lista
    private $dataFinal;         //Data final (admissão) da lista
    private $nrEstrutuLegal;    //Estrutura Legal dos vínculos
    private $nrEstrutuGeren;    //Estrutura Gerencial dos Vícnulos
    private $nrMatriInicial;    //Nro inicial da matrícula
    private $nrMatricuFinal;    //Nro final da matrícula
    private $nrPessResponsa;    //Nro da pessoa responsável
    private $nrEstrResponsa;    //Nro da estrutura da pessoa responsável
    private $nrGrupoInciden;    //Nro do grupo de incidência
    private $nrGrupoIncSaud;    //Nro do grupo de incidência para valores do plano de saúde. Em desuso. É usado no mesmo grupo de incidência, porém no tipo de incidência 99.
    private $idTpInscriResp;    //Tipo de inscrição do responsável (cpf ou cnpj)
    private $idSocioOstensi;    //S sócio ostensivo; N sócio não ostensivo
    private $idDepositJudic;    //S depósito de crédito decorrente de decisão judicial
    private $idInstitInvest;    //S instituição administradora ou intermediária de fundo opu clube de investimento
    private $idPagRendiExte;    //S pagou rendimentos a residentes ou domiciliados no exterior
    private $idPagPlanoSaud;    //S existe pagamento de valor pelo titular/dependente do plano de saúde
    private $idPagRelacCopa;    //S existe pagamento relacionado à Copa
    private $idPagSituaEspe;    //S existe pagamento de situação especial
    private $dtEventoEspeci;    //data do evento especial (se houver)
    private $anoCalendario;
    private $anoReferencia;
    private $nrRecibo;

    private $objMagnetico;

    public function __construct($nrOrg, $nrForm, $dtCompetencia, $nrCenario, $nrOcorrCal, $caminhoRelativo){
        $this->msgErro = null;   //Esta variável nulo significa que não houve erros e nem falta de preenchimento
        $this->carregaDadosFormulario($nrOrg, $nrForm);
        if($this->nrOrg == -1){   //Verifica se o carregamento de dados do formulário funcionou, caso contrário ela sai.
            return;
        }
        switch($this->idOrdem){ //'Zera' os parâmetros não utilizados (necessário para a query)
            case 'ALFABETICA':
                $this->queryPart = <<<SQL
AND ( VM.NMVINCULOM BETWEEN
      (SELECT NMVINCULOM FROM GPE_VINCULOM WHERE NRORG = $this->nrorg AND NRVINCULOM = '$this->nomeInicial') AND
      (SELECT NMVINCULOM FROM GPE_VINCULOM WHERE NRORG = $this->nrorg AND NRVINCULOM = '$this->nomeFinal') )
SQL;
                break;
            case 'DATA_ADMISSAO':
                $this->queryPart = <<<SQL
AND ( VM.DTADMISSAOVINC BETWEEN '$this->dataInicial' /*:P_ADMISSAOINICIAL*/ AND '$this->dataFinal' /*:P_ADMISSAOFINAL*/ )
SQL;
                break;
            case 'MATRICULA':
                $this->queryPart= <<<SQL
AND ( VM.CDMATRICULA BETWEEN '$this->nrMatriInicial' /*:P_MATRICULAINICIAL*/ AND '$this->nrMatricuFinal' /*:P_MATRICULAFINAL*/ )
SQL;
                break;
            case 'ESTRUTURA':
                $this->queryPart = <<<SQL
AND ( VH.NRESTRUTLEGAL = '$this->nrEstrutuLegal' /*:P_ESTRUTURA*/ OR VH.NRESTRUTGEREN = '$this->nrEstrutuGeren' )
SQL;
                break;
            default:
                $this->nrOrg    = -1; //Indica falha
                $this->msgErro .= "Opção de ordenação inválida (".$this->idOrdem.").".Dirf::ql_msg;
        }
        $this->dtCompetencia   = $dtCompetencia;
        $this->nrCenario       = $nrCenario;
        $this->nrOcorren       = $nrOcorrCal;
        $this->caminhoRelativo = $caminhoRelativo.$this->nrOrg.'\\';
        $this->registrosDirf   = array();
        if($this->idPagSituaEspe == 'S' && $this->dtEventoEspeci == null){
            $this->nrOrg    = -1; //Indica falha
            $this->msgErro .= "A situação especial está marcada. Falta a data da situação especial.".Dirf::ql_msg;
        }
        if($this->idPagSituaEspe == 'N')    //Se não há situação especial não deve haver a data da mesma
            $this->dtEventoEspeci = null;
        if($this->idProcessaVinc == 'N' && $this->idGeraMagnetic == 'N'){
            $this->nrOrg    = -1; //Indica falha
            $this->msgErro .= "As opções Gerar Arquivo e Gerar Magnético estão desativadas. Ative uma ou ambas opções para gerar a DIRF.".Dirf::ql_msg;
        }
        if($this->idGeraMagnetic == 'S' && $this->nrOrg != -1)  //Verifica se deve criar o magnético (caso não ocorra erros)
            $this->objMagnetico = new Dirfmagnet($nrOrg, $dtCompetencia);
    }

    //Função que carrega os dados do formulário. Na falha esta seta o nrOrg para -1 indicando a falha
    private function carregaDadosFormulario($nrOrg, $nrForm){
        /** @var $formObj FpaFormdirf  */
        $formulario = new FpaFormdirf();
        $formObj = $formulario->getObjectByWhere('NRORG = ? AND NRFORMDIRF = ?', array($nrOrg, $nrForm));
        //Dados do formulário
        if(is_object($formObj)) {   //Somente grava os dados se encontrou o formulário segundo a Org e seu Nro
            $this->nrOrg          = $nrOrg;
            $this->nrForm         = $nrForm;
            $this->nmForm         = $formObj->getNmformdirf();
            $this->idAtivo        = $formObj->getIdativo();
            $this->idProcessaVinc = $formObj->getIdprocvinculo();
            $this->idGeraMagnetic = $formObj->getIdgerameiomag();
            $this->idExtinsaoEmpr = $formObj->getIdexcempresa();
            $this->idTipoDeclarac = $formObj->getIdtipdeclaracao();
            $this->idNaturezaDecl = $formObj->getIdnaturezadirf();
            $this->idOrdem        = $formObj->getDsordem();
            $this->nomeInicial    = $formObj->getNomeinicial();
            $this->nomeFinal      = $formObj->getNomefinal();
            $this->dataInicial    = $formObj->getDtinicial();
            $this->dataFinal      = $formObj->getDtfinal();
            $this->nrEstrutuLegal = $formObj->getNrestrutlegal();
            $this->nrEstrutuGeren = $formObj->getNrestrutgeren();
            $this->nrMatriInicial = $formObj->getNrmatricinicial();
            $this->nrMatricuFinal = $formObj->getNrmatricfinal();
            $this->nrPessResponsa = $formObj->getNrpessoarespons();
            $this->nrEstrResponsa = $formObj->getNrestrutrespons();
            $this->nrGrupoInciden = $formObj->getNrgrpincide();
            $this->nrGrupoIncSaud = $formObj->getNrgrpincidsaud();
            $this->nrRecibo       = $formObj->getNrregistro();
            $this->idTpInscriResp = $formObj->getIdtpinscrespons();
            $this->idSocioOstensi = $formObj->getIdsociostens();
            $this->idDepositJudic = $formObj->getIddepojudic();
            $this->idInstitInvest = $formObj->getIdinstinvest();
            $this->idPagRendiExte = $formObj->getIdpagrendext();
            $this->idPagPlanoSaud = $formObj->getIdpagplansaud();
            $this->idPagRelacCopa = $formObj->getIdpagrelcopa();
            $this->idPagSituaEspe = $formObj->getIdpagsitespe();
            $this->dtEventoEspeci = $formObj->getDteventespec();
            $this->anoCalendario  = $formObj->getNranocalen();
            $this->anoReferencia  = $formObj->getNranoref();
            //se necessário acrescentar campos do formulário aqui
        } else {    //Formulário não encontrado
            $this->nrOrg  = -1; //indica que não carregou os dados
            $this->msgErro = 'Formulário não carregado. O número do formulário '.$nrForm.' da organização número '.$nrOrg.' não foi encontrado.'.Dirf::ql_msg;
        }
    }

    //COLEÇÃO DE REGISTROS. CADA MÉTODO GERA UM REGISTRO DO DIRF. VER MANUAL DA DIRF PARA MAIOR ESCLARECIMENTO.

    //DIRF 3.1 Declaração do Imposto sobre a renda retido na fonte
    private function _dirf(){
        if(is_null($this->anoReferencia) || $this->anoReferencia == '')
            $this->msgErro .= "Está faltando o ano de referência.".Dirf::ql_msg;
        if(is_null($this->anoCalendario) || $this->anoCalendario == '')
            $this->msgErro .= "Está faltando o ano de calendário.".Dirf::ql_msg;
        if($this->idTipoDeclarac != 'S' && $this->idTipoDeclarac != 'N' )
            $this->msgErro .= "Está faltando a indicação do tipo de declaração.".Dirf::ql_msg;
        $anoReferencia  = str_pad(substr($this->anoReferencia,  0,  4),  4,' '); //Garante o tamanho do campo em 4
        $anoCalendario  = str_pad(substr($this->anoCalendario,  0,  4),  4,' '); //Garante o tamanho do campo em 4
        $idTipoDeclarac = str_pad(substr($this->idTipoDeclarac, 0,  1),  1,' '); //Garante o tamanho do campo em 1
        $auxNrRecibo    = str_pad(substr($this->nrRecibo,       0, 12), 12,' '); //Garante o tamanho do campo em 12
        return 'DIRF|'.$anoReferencia.'|'.$anoCalendario.'|'.$idTipoDeclarac.'|'.$auxNrRecibo.'|F8UCL6S|';
    }

    //RESPO 3.2 Responsável pelo preenchimento
    private function _respo(){
        $sql = <<<SQL
SELECT
  PE.NRPESSOA,
  'RESPO|'                                   NMIDENTIFICADOR,
  LPAD(PEH.NRCPFPESSOA, 11, '0')      || '|' NRCPF,
  SUBSTR(PEH.NMPESSOA, 1, 60)         || '|' NMNOMERESP,
  SUBSTR( TRIM( LEADING '0' FROM REPLACE( TRANSLATE( TEL.CDPREFIXCOMUPARC, '() ', '-'), '-', '') ), 1, 2 ) || '|' NRDDD,
  SUBSTR(                        REPLACE( TRANSLATE( TEL.CDCOMUNICAPARC,   '() ', '-'), '-', '')  , 1, 9 ) || '|' NRTELEFONE,
  '|'                                        NRRAMAL,
  SUBSTR( REPLACE( TRANSLATE( FAX.CDCOMUNICAPARC, '() ', '-'), '-', ''), 1, 9 )                            || '|' NRFAX,
  SUBSTR( EMA.CDCOMUNICAPARC, 1, 50 ) || '|' NMEMAIL
FROM GPE_PESSOA PE
LEFT JOIN GPE_PESSOAH PEH ON PEH.NRORG        = PE.NRORG
                         AND PEH.NRPESSOA     = PE.NRPESSOA
--Telefone
LEFT JOIN COMUNICAPARC TEL ON TEL.NRORG         = PE.NRORG
                          AND TEL.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                          AND TEL.CDFORMACOMU   = 1 --Forma de comunicação: telefone
--Fax
LEFT JOIN COMUNICAPARC FAX ON FAX.NRORG         = PE.NRORG
                          AND FAX.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                          AND FAX.CDFORMACOMU   = 10 --Forma de comunicação: fax
--Correio Eletrônico
LEFT JOIN COMUNICAPARC EMA ON EMA.NRORG         = PE.NRORG
                          AND EMA.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                          AND EMA.CDFORMACOMU   = 5 --Forma de comunicação: e-mail
---
WHERE PE.NRORG    = $this->nrOrg /*:P_NRORG*/
  AND PE.NRPESSOA = $this->nrPessResponsa /*:P_NRPESSOARESP*/ --Responsável informal
  --Pessoa
  AND ( PEH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                               WHERE DTMESCOMPETENC <= '$this->dtCompetencia' /*:P_COMPETENCIA*/
                                 AND NRORG    = PE.NRORG
                                 AND NRPESSOA = PE.NRPESSOA )
        OR PEH.DTMESCOMPETENC IS NULL )
  --Telefone
  AND ( TEL.NRCOMUNICAPARC = ( SELECT MAX(NRCOMUNICAPARC) FROM COMUNICAPARC
                               WHERE NRORG         = PE.NRORG
                                 AND NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                                 AND CDFORMACOMU   = 1 )
        OR TEL.NRCOMUNICAPARC IS NULL )
  --Fax
  AND ( FAX.NRCOMUNICAPARC = ( SELECT MAX(NRCOMUNICAPARC) FROM COMUNICAPARC
                               WHERE NRORG         = PE.NRORG
                                 AND NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                                 AND CDFORMACOMU   = 10 )
        OR FAX.NRCOMUNICAPARC IS NULL )

  AND ( EMA.NRCOMUNICAPARC = ( SELECT MAX(NRCOMUNICAPARC) FROM COMUNICAPARC
                               WHERE NRORG         = PE.NRORG
                                 AND NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                                 AND CDFORMACOMU   = 5 )
        OR EMA.NRCOMUNICAPARC IS NULL )
SQL;
        try{
            $sqlResult = \Setforms2\Runtime\Framework::getInstance()->fetchAll($sql);
        } catch(\Exception $e){
            throw new \Exception('Erro na query, campo RESPO, item 3.2, a query usada foi esta: '.$sql);
        }
        $registro = '';     //Armazena o registro
        foreach($sqlResult as $val){
            if($val['NRCPF']      == '|')   //'IFs' para verificação de campos não preenchidos
                $this->msgErro .= "Responsável pela DIRF sem CPF.".Dirf::ql_msg;
            if($val['NMNOMERESP'] == '|')
                $this->msgErro .= "Responsável pela DIRF sem nome.".Dirf::ql_msg;
            if($val['NRDDD']      == '|')
                $this->msgErro .= "Responsável pela DIRF sem Nº de DDD.".Dirf::ql_msg;
            if($val['NRTELEFONE'] == '|')
                $this->msgErro .= "Responsável pela DIRF sem Nº de telefone.".Dirf::ql_msg;
            $registro = $val['NMIDENTIFICADOR'].$val['NRCPF'].$val['NMNOMERESP'].$val['NRDDD'].$val['NRTELEFONE'].$val['NRRAMAL'].$val['NRFAX'].$val['NMEMAIL'];

            //todo colocar este no lugar certo e parâmetrizado
            if($this->idGeraMagnetic == 'S')
                $this->objMagnetico->addRegCabecal($val['CDINTESTRUTURA'], null, $val['NRPESSOA'], $this->idAtivo);
        }
        return $registro;
    }

    //DECPJ 3.4 Registro de identificação do declarante pessoa juridica
    private function _decpj(){
        $sql = <<<SQL
SELECT
  'DECPJ|'                                     NMIDENTIFICADOR,
  LPAD( SUBSTR( EH.CDCNPJESTRUT, 1, 14), 14, '0' ) || '|' CDCNPJ,
  SUBSTR( EH.NMRAZSOCESTRUT, 1, 150 )   || '|' NOMNEEMPRESARIAL,
  '$this->idNaturezaDecl'               || '|' IDNATUREZADECLA,
  LPAD(PEH.NRCPFPESSOA, 11, '0')        || '|' CDCPFRESP,
  '$this->idSocioOstensi'               || '|' IDSOCIOSTENS,
  '$this->idDepositJudic'               || '|' IDDEPOJUDIC,
  '$this->idInstitInvest'               || '|' IDINSTINVEST,
  '$this->idPagRendiExte'               || '|' IDPAGRENDEXT,
  '$this->idPagPlanoSaud'               || '|' IDPAGPLANSAUD,
  '$this->idPagRelacCopa'               || '|' IDPAGRELCOPA,
  '$this->idPagSituaEspe'               || '|' IDPAGSITESPE,
  TO_CHAR( '$this->dtEventoEspeci', 'YYYYMMDD') || '|' DTEVENTDECPJ
FROM ESTRUTURAH EH
--Responsável informal (Pessoa e Parceiro de Negócio)
LEFT JOIN GPE_PESSOA   PE ON PE.NRORG         = EH.NRORG
                         AND PE.NRPARCNEGOCIO = EH.NRPARCENTISIN  --Responsável legal
LEFT JOIN GPE_PESSOAH PEH ON PEH.NRORG        = EH.NRORG
                         AND PEH.NRPESSOA     = PE.NRPESSOA
---
WHERE EH.NRORG        = $this->nrOrg --:P_NRORG
  AND EH.NRESTRUTURAM = $this->nrEstrResponsa --:P_NRESTRUTRESP
  --Estrutura
  AND EH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                            WHERE DTMESCOMPETENC <= '$this->dtCompetencia' --:P_COMPETENCIA
                              AND NRORG        = EH.NRORG
                              AND NRESTRUTURAM = EH.NRESTRUTURAM )
SQL;
        try{
            $sqlResult = \Setforms2\Runtime\Framework::getInstance()->fetchAll($sql);
        } catch(\Exception $e){
            throw new \Exception('Erro na query, campo DECPJ, item 3.4, a query usada foi esta: '.$sql);
        }
        $registro = '';
        foreach($sqlResult as $val){
            if($val['CDCNPJ']          == '|')   //'IFs' para verificação de campos não preenchidos
                $this->msgErro .= "Estrutura do responsável pela DIRF (nº ".$this->nrEstrResponsa.") sem CNPJ.".Dirf::ql_msg;
            if($val['NOMEEMPRESARIAL'] == '|')
                $this->msgErro .= "Estrutura do responsável pela DIRF (nº ".$this->nrEstrResponsa.") sem razão social.".Dirf::ql_msg;
            if($val['IDNATUREZADECLA'] == '|')
                $this->msgErro .= "Não há natureza do declarante (formulário, estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['CDCPFRESP']       == '|')
                $this->msgErro .= "Responsável perante o CNPJ não cadastrado ou sem CPF (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDSOCIOSTENS']    == '|')
                $this->msgErro .= "Falta indicador de sócio ostensivo (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDDEPOJUDIC']     == '|')
                $this->msgErro .= "Falta indicador de depósito de crédito decorrente de decisão judicial (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDINSTINVEST']    == '|')
                $this->msgErro .= "Falta indicador de declarante de instituição administradora (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDPAGRENDEXT']    == '|')
                $this->msgErro .= "Falta indicador de rendimentos pagos a residentes no exterior (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDPAGPLANSAUD']   == '|')
                $this->msgErro .= "Falta indicador de plano privado de assistência à saúde (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDPAGRELCOPA']    == '|')
                $this->msgErro .= "Falta indicador de pagamentos relacionados à Copa das Confedêrações e do Mundo (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDPAGSITESPE']    == '|')
                $this->msgErro .= "Falta indicador de situação especial de declaração (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDPAGSITESPE'] == '|S' && $val['DTEVENTDECPJ'] == '|')
                $this->msgErro .= "Falta data de situação especial de declaração (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            foreach ($val as $res) {
                $registro .= $res;
            }
        }
        return $registro;
    }

    //IDREC 3.5, BPFDEC 3.6, 4.6 campos de registros
    private function _motor(){
        $idDecTerceiro      = self::id13Sal;
        $queryPartCdReceita = self::queryPartCdReceita;
        $idInforme          = self::idInforme;

        $sql = <<<SQL
SELECT
  --PRIN.NRVINCULOM,
  PRIN.NRCPFPESSOA,
  --PRIN.NMVINCULOM,
  PRIN.NMPESSOA,
  --PRIN.DTADMISSAOVINC,
  PRIN.NMITEM,
  CASE  --Verifica se o registro pode ser relacionado no TXT da DIRF
    WHEN INSTR(PRIN.NMITEM, '$idInforme'/*'-INF'*/) > 0 THEN 'N' --Não aparece no TXT da DIRF
    ELSE 'S'
  END FLAG_TXT,
  ROUND( NVL(PRIN.M01, 0) + NVL(PRIN.M02, 0) + NVL(PRIN.M03, 0) + NVL(PRIN.M04, 0) + NVL(PRIN.M05, 0) + NVL(PRIN.M06, 0) + NVL(PRIN.M07, 0) + NVL(PRIN.M08, 0) + NVL(PRIN.M09, 0) + NVL(PRIN.M10, 0) + NVL(PRIN.M11, 0) + NVL(PRIN.M12, 0) /*+ NVL(M13, 0)*/, 2 ) TOTAL,
  ROUND( NVL(PRIN.M13, 0), 2 ) TOTAL13,
  PRIN.IDREC, --3.5
  PRIN.CDRECEITA,
  PRIN.BPFDEC, --3.6
  PRIN.CDCPF,
  PRIN.NOME,
  PRIN.DTMOLESTIA,
  PRIN.CDREGISTRO, --4.6
  SUBSTR( ( ROUND(PRIN.M01, 2) * 100 ), 1, 13 ) || '|' M01,
  SUBSTR( ( ROUND(PRIN.M02, 2) * 100 ), 1, 13 ) || '|' M02,
  SUBSTR( ( ROUND(PRIN.M03, 2) * 100 ), 1, 13 ) || '|' M03,
  SUBSTR( ( ROUND(PRIN.M04, 2) * 100 ), 1, 13 ) || '|' M04,
  SUBSTR( ( ROUND(PRIN.M05, 2) * 100 ), 1, 13 ) || '|' M05,
  SUBSTR( ( ROUND(PRIN.M06, 2) * 100 ), 1, 13 ) || '|' M06,
  SUBSTR( ( ROUND(PRIN.M07, 2) * 100 ), 1, 13 ) || '|' M07,
  SUBSTR( ( ROUND(PRIN.M08, 2) * 100 ), 1, 13 ) || '|' M08,
  SUBSTR( ( ROUND(PRIN.M09, 2) * 100 ), 1, 13 ) || '|' M09,
  SUBSTR( ( ROUND(PRIN.M10, 2) * 100 ), 1, 13 ) || '|' M10,
  SUBSTR( ( ROUND(PRIN.M11, 2) * 100 ), 1, 13 ) || '|' M11,
  SUBSTR( ( ROUND(PRIN.M12, 2) * 100 ), 1, 13 ) || '|' M12,
  SUBSTR( ( ROUND(PRIN.M13, 2) * 100 ), 1, 13 ) || '|' M13,
  COMP.COMPLEMENTO
FROM (
  SELECT * FROM ( --este select é para o pivo funcionar
    SELECT
      --VM.NRVINCULOM,
      --VM.NMVINCULOM,
      PH.NRCPFPESSOA,
      PH.NMPESSOA,
      --VM.DTADMISSAOVINC,
      REPLACE(GIT.NMITEM, '$idDecTerceiro'/*'-13S'*/, '') NMITEM, --Remove a idêntificação do 13º salário (este vai para o M13 e VALOR13)
      --GIT.NRITEM,
      'IDREC|'                             IDREC, --3.5
      CASE
        $queryPartCdReceita/*WHEN ICF.NREVENTOM = 870 THEN '3562|'*/
        /*WHEN VM.NRTIPOVINCULOM = 2 THEN '0588|'*/
        ELSE LPAD(VE.CODRECEITA, 4, '0')   || '|'
      END CDRECEITA,
      'BPFDEC|'                            BPFDEC, --3.6
      LPAD(PH.NRCPFPESSOA, 11, '0') || '|' CDCPF,
      SUBSTR(PH.NMPESSOA, 1, 60)    || '|' NOME,
      '|'                                  DTMOLESTIA,
      REPLACE(GIT.NMITEM, '$idDecTerceiro'/*'-13S'*/, '') || '|' CDREGISTRO, --4.6
      CASE
        WHEN INSTR(GIT.NMITEM, '$idDecTerceiro'/*'-13S'*/) > 0 THEN '13' --Verifica se o item é do 13º salário (Ex.: se o id13Sal for '-13S' então 'RTRT-13S' é o M13 do 'RTRT')
        ELSE TO_CHAR(CF.DTOCORRENCIA, 'mm')
      END MES, --são as colunas do pivô
      --ICF.VREVENTOFOLHA --Original 12/02/2015
      DECODE(IEV.IDINCIDEVENTO, '-', -1 * ICF.VREVENTOFOLHA, ICF.VREVENTOFOLHA) VREVENTOFOLHA --Considera o '+' e '-' no sum
    FROM GPE_VINCULOM VM
    JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                        AND VH.NRVINCULOM = VM.NRVINCULOM
    --Código da receita do vínculo em questão (item 3.5)
    LEFT JOIN GPE_VINCULOEMPREG VE ON ( VE.NRORG IS NULL OR VE.NRORG = 0 OR VE.NRORG = VM.NRORG )
                                  AND VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
    --CPF e Nome do vínculo em questão (item 3.6)
    JOIN GPE_PESSOA  PE ON PE.NRORG    = VM.NRORG
                       AND PE.NRPESSOA = VM.NRPESSOA
    JOIN GPE_PESSOAH PH ON PH.NRORG    = VM.NRORG
                       AND PH.NRPESSOA = PE.NRPESSOA
    --Grupo de incidência da DIRF, Itens de Grupo e suas Incidências de eventos e os Itens de Cálculo
    JOIN FPA_GRPITEM      GIT ON ( GIT.NRORG = VM.NRORG OR GIT.NRORG = 0 )
                             AND GIT.NRGRPINCIDE  = $this->nrGrupoInciden/*:P_NRGRPDIRF*/
    JOIN FPA_INCIDEVENTO  IEV ON ( IEV.NRORG = VM.NRORG OR IEV.NRORG = 0 )
                             AND IEV.NRTIPOINCIDE = GIT.NRTIPOINCIDE
    JOIN FPA_CALCULOFOLHA  CF ON CF.NRORG          = VM.NRORG
                             --AND CF.NRTIPOMOVIMENT IN ( 1, /*4,*/ 2 ) --Movimentos mensal, 13º salário E FÉRIAS
                             AND ( CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RIAP', 2, 1) OR CF.NRTIPOMOVIMENT = DECODE(GIT.NMITEM, 'RTRT$idDecTerceiro', 4, 'RTRT$idDecTerceiro$idInforme', 4, 'RTPO$idDecTerceiro', 4, 'RTIRF$idDecTerceiro', 4) )--O RIAP (abono pencuniário) somente é considerado no movimento férias (2). Demias são no movimento mensal (1)
                             AND CF.NRTPMODALIDCAL = $this->nrCenario/*:P_CENARIO*/ --Cenário real apenas
                             --AND CF.NROCORRECAL    = $this->nrOcorren /*:P_OCORRENCIACALC*/
                             AND CF.DTOCORRENCIA BETWEEN TRUNC( TO_DATE( '$this->dtCompetencia'/*:P_COMPETENCIA*/ ), 'YEAR' ) AND '$this->dtCompetencia'/*:P_COMPETENCIA*/
    JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG          = VM.NRORG
                             AND ICF.NRVINCULOM     = VM.NRVINCULOM
                             AND ICF.NREVENTOM      = IEV.NREVENTOM
                             AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
    WHERE VM.NRORG = $this->nrOrg/*:P_NRORG*/
      AND ( VM.DTRESCISAOVINC IS NULL
            OR ( VM.DTRESCISAOVINC >= TRUNC( TO_DATE( '$this->dtCompetencia'/*:P_COMPETENCIA*/ ), 'YEAR' ) ) )
      AND GIT.NRITEM != 99 --Todos os itens, exceto o 'informações complementares'
      /*$this->queryPart*/
      --Pessoa
      AND PH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                WHERE DTMESCOMPETENC <= '$this->dtCompetencia'/*:P_COMPETENCIA*/
                                  AND NRORG    = VM.NRORG
                                  AND NRPESSOA = PE.NRPESSOA )
      --Vínculo
      AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                WHERE DTMESCOMPETENC <= '$this->dtCompetencia'/*:P_COMPETENCIA*/
                                  AND NRORG      = VM.NRORG
                                  AND NRVINCULOM = VM.NRVINCULOM )
  ) PIVOT ( SUM(VREVENTOFOLHA)
            FOR MES IN ('01' AS M01, '02' AS M02, '03' AS M03, '04' AS M04, '05' AS M05, '06' AS M06, '07' AS M07, '08' AS M08, '09' AS M09, '10' AS M10, '11' AS M11, '12' AS M12, '13' AS M13) )
) PRIN
LEFT JOIN ( SELECT
              --RES.NRVINCULOM,
              RES.NMVINCULOM,
              LISTAGG(RES.NMEVENTOH || ': ' || RES.VALOR, CHR(13)) WITHIN GROUP (ORDER BY RES.NMEVENTOH) COMPLEMENTO
            FROM (SELECT
                    --VM.NRVINCULOM,
                    VM.NMVINCULOM,
                    EVH.NMEVENTOH,
                    TO_CHAR(SUM(ICF.VREVENTOFOLHA), '99999.99') VALOR
                  FROM GPE_VINCULOM VM
                  JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                                      AND VH.NRVINCULOM = VM.NRVINCULOM
                  --Grupo de incidência da DIRF, Itens de Grupo e suas Incidências de eventos e os Itens de Cálculo
                  JOIN FPA_GRPINCIDE    GIN ON ( GIN.NRORG = VM.NRORG OR GIN.NRORG = 0 )
                                           AND GIN.NRGRPINCIDE = $this->nrGrupoInciden/*:P_NRGRPDIRF*/
                  JOIN FPA_GRPITEM      GIT ON ( GIT.NRORG = VM.NRORG OR GIT.NRORG = 0 )
                                           AND GIT.NRGRPINCIDE = GIN.NRGRPINCIDE
                  JOIN FPA_INCIDEVENTO  IEV ON ( IEV.NRORG = VM.NRORG OR IEV.NRORG = 0 )
                                           AND IEV.NRTIPOINCIDE = GIT.NRTIPOINCIDE
                  JOIN FPA_CALCULOFOLHA  CF ON CF.NRORG          = VM.NRORG
                                           AND CF.NRTIPOMOVIMENT = 1 --Movimentos mensal
                                           AND CF.NRTPMODALIDCAL = $this->nrCenario/*:P_CENARIO*/ --Cenário real apenas
                                           --AND CF.NROCORRECAL    = $this->nrOcorren/*:P_OCORRENCIACALC*/
                                           AND CF.DTOCORRENCIA BETWEEN TRUNC( TO_DATE( '$this->dtCompetencia'/*:P_COMPETENCIA*/ ), 'YEAR' ) AND '$this->dtCompetencia'/*:P_COMPETENCIA*/
                  JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG          = VM.NRORG
                                           AND ICF.NRVINCULOM     = VM.NRVINCULOM
                                           AND ICF.NREVENTOM      = IEV.NREVENTOM
                                           AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                  JOIN FPA_EVENTOH EVH ON ( EVH.NRORG = 0 OR EVH.NRORG = VM.NRORG )
                                      AND EVH.NREVENTOM = IEV.NREVENTOM
                                      AND EVH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM FPA_EVENTOH
                                                                 WHERE NREVENTOM = IEV.NREVENTOM
                                                                   AND DTMESCOMPETENC <= '$this->dtCompetencia'/*:P_COMPETENCIA*/
                                                                   AND ( NRORG = 0 OR NRORG = VM.NRORG ) )
                  WHERE VM.NRORG = $this->nrOrg/*:P_NRORG*/
                    AND ( VM.DTRESCISAOVINC IS NULL OR ( VM.DTRESCISAOVINC >= TRUNC( TO_DATE( '$this->dtCompetencia'/*:P_COMPETENCIA*/ ), 'YEAR' ) ) )
                    AND GIT.NRITEM = 99 --Apenas 'informações complementares'
                    /*$this->queryPart*/
                    --Vínculo
                    AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                              WHERE DTMESCOMPETENC <= '$this->dtCompetencia'/*:P_COMPETENCIA*/
                                                AND NRORG      = VM.NRORG
                                                AND NRVINCULOM = VM.NRVINCULOM )
                  GROUP BY /*VM.NRVINCULOM*/ VM.NMVINCULOM, EVH.NMEVENTOH
            ) RES
            GROUP BY RES.NMVINCULOM --RES.NRVINCULOM
) COMP ON COMP.NMVINCULOM = PRIN.NOME --COMP.NRVINCULOM = PRIN.NRVINCULOM
ORDER BY CDRECEITA, CDCPF--, NRITEM
SQL;
        try{
            $sqlResult = \Setforms2\Runtime\Framework::getInstance()->fetchAll($sql);
            //$sqlResult = \Setforms2\Shared\Connection::getInstance()->fetchAll($sql);
        } catch(\Exception $e){
            throw new \Exception('Erro na query, campos IDREC, BPFDEC e campos de registros, itens 3.5, 3.6 e 4.6, a query usada foi esta: '.$sql);
        }
        if(count($sqlResult) == 0){ //Testa se existe resultado
            $this->msgErro .= "Não há registros para os parâmetros selecionados.".Dirf::ql_msg;
            //return null;
        }
        $idrecAntes      = '';      //Armazena os valores anteriores
        $bpfdecAntes     = '';      //Armazena os valores anteriores
        $registrosDirf   = array(); //Registros para o arquivo da DIRF
        $dsLinhaRegistro = '';      //Para alimentar o magnético

        foreach($sqlResult as $val){    //Laço para capturar todos os registros e formar
//            if($val['FLAG_TXT'] == 'N') //Verifica se o registro pode ser relacionado no TXT da DIRF.
//                continue;
            if($val['FLAG_TXT'] == 'S') {
                $idrec = $val['IDREC'] . $val['CDRECEITA'];
                $bpfdec = $val['BPFDEC'] . $val['CDCPF'] . $val['NOME'] . $val['DTMOLESTIA'];
                $reg = $val['CDREGISTRO'] . $val['M01'] . $val['M02'] . $val['M03'] . $val['M04'] . $val['M05'] . $val['M06'] . $val['M07'] . $val['M08'] . $val['M09'] . $val['M10'] . $val['M11'] . $val['M12'] . $val['M13'];
                if ($idrec != $idrecAntes) {          //Verifica se houve novo registro do tipo IDREC
                    $idrecAntes = $idrec;          //Atualiza o IDREC corrente
                    $bpfdecAntes = '';              //Para garantir que o mesmo gere uma linha BPFDEC
                    array_push($registrosDirf, $idrec); //Adiciona o registro IDREC (somente uma vez)
                }
                if ($bpfdec != $bpfdecAntes) {            //Verifica se houve um novo registro do tipo BPFDEC
                    if ($val['CDCPF'] == '|')    //'IFs' para verificação de campos não preenchidos
                        $this->msgErro .= "Vínculo " . $val['NMPESSOA'] . " sem CPF." . Dirf::ql_msg;   //$this->msgErro .= "Vínculo (".$val['NRVINCULOM'].", ".$val['NMVINCULOM'].") sem CPF.".Dirf::ql_msg;
                    if ($val['NOME'] == '|')
                        $this->msgErro .= "Vínculo de CPF " . $val['NRCPFPESSOA'] . " sem nome." . Dirf::ql_msg;  //$this->msgErro .= "Vínculo ".$val['NRVINCULOM']." sem nome.".Dirf::ql_msg;
                    if ($val['CDRECEITA'] == '|')
                        $this->msgErro .= "Vínculo " . $val['NMPESSOA'] . " de CPF " . $val['NRCPFPESSOA'] . " sem código de receita DIRF ou sem vínculo empregatício." . Dirf::ql_msg;
                    $bpfdecAntes = $bpfdec;             //Atualiza o BPFDEC corrente
                    array_push($registrosDirf, $bpfdec);    //Adiciona o registro BPFDEC (somente uma vez)
                }
                //            if(strpos($val['CDREGISTRO'], self::idInforme) !== false ){ //Verifica se o registro pode ser relacionado no TXT da DIRF
                //                continue;
                //            }
                array_push($registrosDirf, $reg);       //Adiciona o registro
            }
            if($this->idGeraMagnetic == 'S' && $this->msgErro == null) { //Somente gera o final do magnético se o mesmo foi solicitado, se existem registros e não houver erros
                $dsLinhaRegistro = $idrecAntes.Dirf::ql_dslinha.$bpfdecAntes.Dirf::ql_dslinha.$reg;
                $this->objMagnetico->addRegVinculo($val['NRCPFPESSOA'], $val['NMPESSOA'], null, $val['NMITEM'], $val['CDRECEITA'], $val['TOTAL'], $val['TOTAL13'], $dsLinhaRegistro, $val['COMPLEMENTO']);
            }
        }
        return $registrosDirf;
    }

    //PSE 5 Registro de pagamentos a plano privado de assitência à saúde - coletivo empresarial
    private function _pse(){
        return 'PSE|';
    }

    //INF 5.7 Registro de informações complementares para o comprovante de rendimento (baiscamente o plano de saúde)
    private function _motor_inf(){
        $sql = <<<SQL
SELECT
  --VM.NRVINCULOM,
  VM.NMVINCULOM,
  'INF|'                               INF,
  LPAD(PH.NRCPFPESSOA, 11, '0') || '|' CDCPF,
  'DESPESAS MEDICOODONTOHOSPITALARES : ' || TO_CHAR(SUM(ICF.VREVENTOFOLHA), '99999999999.99') || '|' COMPLEMENTO
  --SUBSTR(/*GIN.NMGRPINCIDE*/EVH.NMEVENTOH, 1, 184) || ': ' || TO_CHAR(SUM(ICF.VREVENTOFOLHA), '99999999999.99') || '|' COMPLEMENTO
FROM GPE_VINCULOM VM
JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                    AND VH.NRVINCULOM = VM.NRVINCULOM
--CPF e Nome do vínculo em questão (item 3.6)
JOIN GPE_PESSOA  PE ON PE.NRORG    = VM.NRORG
                   AND PE.NRPESSOA = VM.NRPESSOA
JOIN GPE_PESSOAH PH ON PH.NRORG    = VM.NRORG
                   AND PH.NRPESSOA = PE.NRPESSOA
--Grupo de incidência da DIRF, Itens de Grupo e suas Incidências de eventos e os Itens de Cálculo
JOIN FPA_GRPINCIDE    GIN ON ( GIN.NRORG = VM.NRORG OR GIN.NRORG = 0 )
                         AND GIN.NRGRPINCIDE = $this->nrGrupoInciden /*nrGrupoIncSaud*/ /*:P_NRGRPDIRFSAUDE*/
JOIN FPA_GRPITEM      GIT ON ( GIT.NRORG = VM.NRORG OR GIT.NRORG = 0 )
                         AND GIT.NRGRPINCIDE = GIN.NRGRPINCIDE
JOIN FPA_INCIDEVENTO  IEV ON ( IEV.NRORG = VM.NRORG OR IEV.NRORG = 0 )
                         AND IEV.NRTIPOINCIDE = GIT.NRTIPOINCIDE
JOIN FPA_CALCULOFOLHA  CF ON CF.NRORG          = VM.NRORG
                         AND CF.NRTIPOMOVIMENT = 1 --Movimentos mensal
                         AND CF.NRTPMODALIDCAL = $this->nrCenario /*:P_CENARIO*/ --Cenário real apenas
                         --AND CF.NROCORRECAL    = $this->nrOcorren /*:P_OCORRENCIACALC*/
                         AND CF.DTOCORRENCIA BETWEEN TRUNC( TO_DATE( '$this->dtCompetencia' /*:P_COMPETENCIA*/ ), 'YEAR' ) AND '$this->dtCompetencia' /*:P_COMPETENCIA*/
JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG          = VM.NRORG
                         AND ICF.NRVINCULOM     = VM.NRVINCULOM
                         AND ICF.NREVENTOM      = IEV.NREVENTOM
                         AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
JOIN FPA_EVENTOH EVH ON ( EVH.NRORG = 0 OR EVH.NRORG = VM.NRORG )
                    AND EVH.NREVENTOM = IEV.NREVENTOM
                    AND EVH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM FPA_EVENTOH
                                               WHERE NREVENTOM = IEV.NREVENTOM
                                                 AND DTMESCOMPETENC <= '$this->dtCompetencia' /*:P_COMPETENCIA*/
                                                 AND ( NRORG = 0 OR NRORG = VM.NRORG ) )
WHERE VM.NRORG = $this->nrOrg /*:P_NRORG*/
  AND ( VM.DTRESCISAOVINC IS NULL OR ( VM.DTRESCISAOVINC >= TRUNC( TO_DATE( '$this->dtCompetencia'/*:P_COMPETENCIA*/ ), 'YEAR' ) ) )
  AND GIT.NRITEM = 99 --Apenas 'informações complementares'
  /*$this->queryPart*/
  --Pessoa
  AND PH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                            WHERE DTMESCOMPETENC <= '$this->dtCompetencia' /*:P_COMPETENCIA*/
                              AND NRORG    = VM.NRORG
                              AND NRPESSOA = PE.NRPESSOA )
  --Vínculo
  AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                            WHERE DTMESCOMPETENC <= '$this->dtCompetencia' /*:P_COMPETENCIA*/
                              AND NRORG      = VM.NRORG
                              AND NRVINCULOM = VM.NRVINCULOM )
GROUP BY /*VM.NRVINCULOM, */VM.NMVINCULOM, PH.NRCPFPESSOA/*, EVH.NMEVENTOH*/
ORDER BY PH.NRCPFPESSOA
SQL;
        try{
            $sqlResult = \Setforms2\Runtime\Framework::getInstance()->fetchAll($sql);
        } catch(\Exception $e){
            throw new \Exception('Erro na query, campo INF, iten 5.7, a query usada foi esta: '.$sql);
        }
        $registros = array();
        foreach($sqlResult as $val){
            if($val['CDCPF']      == '|')   //'IFs' para verificação de campos não preenchidos
                $this->msgErro .= "Vínculo (".$val['NMVINCULOM'].") sem CPF.".Dirf::ql_msg;
            array_push($registros, $val['INF'].$val['CDCPF'].$val['COMPLEMENTO']);
        }
        return $registros;
    }

    //FIMDirf 5.8 Registro indicador do término da declaração
    private function _fimdirf(){
        return 'FIMDIRF|';
    }

    //FIM DA COLEÇÃO DE REGISTROS

    private function geraRegistrosDirf(){   //Gera em um vetor os registros da DIRF para o arquivo e o magnético
        tk_disable_result_set_limit();      //Desliga o limite de rows. Os métodos _motor() e _motor_inf() geralmente geram mais de 300 rows.
        array_push($this->registrosDirf, $this->_dirf());    //DIRF 3.1 Declaração do Imposto sobre a renda retido na fonte
        array_push($this->registrosDirf, $this->_respo());   //RESPO 3.2 Responsável pelo preenchimento
        array_push($this->registrosDirf, $this->_decpj());   //DECPJ 3.4 Registro de identificação do declarante pessoa juridica
        foreach($this->_motor() as $reg){                    //IDREC 3.5, BPFDEC 3.6, 4.6 campos de registros
            array_push($this->registrosDirf, $reg);
        }
        if($this->idGeraMagnetic == 'S' && $this->msgErro == null) { //Somente gera o final do magnético se o mesmo foi solicitado, se existem registros e não houver erros
            $this->objMagnetico->fimAddRegVinculo();
        }
        foreach($this->_motor_inf() as $reg){                 //INF 5.7 Registro de informações complementares para o comprovante de rendimento
            array_push($this->registrosDirf, $reg);
        }
        array_push($this->registrosDirf, $this->_fimdirf()); //FIMDirf 5.8 Registro indicador do término da declaração
    }

    public function gerar() { //Gera o DIRF
        if($this->nrOrg == -1) //Verifica se o formulário foi carregado ou se não houve outras falhas / inconsistências
            return false;

        $this->geraRegistrosDirf(); //Gera os registros da dirf E DO MAGNÉTICO (se for o caso)

        if(!is_null($this->msgErro))    //Verifica se não houve quaisquer inconsistências durante a geração dos registros da DIRF
            return false;               //Houve incosistência, aborta a geração.

        if($this->idProcessaVinc == 'S'){  //Verifica se deve criar o arquivo txt da DIRF
            if(!file_exists($this->caminhoRelativo.Dirf::nomeArquivo)){       //Se não existir a estrutura de pastas este a criará
                mkdir($this->caminhoRelativo, 07777, true);
            }
            foreach (glob($this->caminhoRelativo.'*.*') as $arquivo) {        //Deleta os arquivos da pasta 'caminhoRelativo' + o número da organização
                unlink($arquivo);
            }
            $arquivo = fopen($this->caminhoRelativo.Dirf::nomeArquivo, 'w-'); //Cria o arquivo para armazenar a dirf (no 'caminhoRelativo' + nro da org + 'nomeArquvivo')
            foreach($this->registrosDirf as $reg){                            //Adiciona os registro no arquivo
                fwrite($arquivo, $reg.PHP_EOL);
            }
            fclose($arquivo);                                                 //Fecha o arquivo
            $arquivo = new \TkFile($this->caminhoRelativo.Dirf::nomeArquivo); //variável para o download do arquivo
            $arquivo->download();                                             //Inicia o download do arquivo
        }

        if($this->idGeraMagnetic == 'S'){   //Verifica se o magnético foi gerado para emitir o relatório
            $param = array();
            $param['P_COMPETENCIA'] = \FPA::getDtCompetencia();
            $param['P_NRORG'] = \FPA::getNrorgTrabalho();
            $param['P_ANOREF'] = tk_get_field_value('NRANOREF');
            $param['P_ANOCALEN'] = tk_get_field_value('NRANOCALEN');
            $param['P_CONFIDENCIAL'] = \Folha\Facade\Relatorios\Confidencial::setQueryConfidencial();
            $param['CDOPERADOR'] = \Geral\Util\Ambiente::getCdOperador();
            $param['NMOPERADOR'] = \Geral\Util\Ambiente::getNmOperador();
            tk_open_report_from_array('informe_rendimentos', $param, \Setforms2\Runtime\Helper\Report\TkBirtReport::FORMAT_PDF);
        }

        return true;
    }

    public function retroceder(){
        if($this->nrOrg == -1) {    //Verifica se o formulário foi carregado
            $this->msgErro .= "Não foi possível retroceder a Dirf pois o formulário não está carregado.".Dirf::ql_msg;
            return false;           //Retorna erro e não retrocede
        }
        $cont = $this->objMagnetico->excluirCompetencia(/*$this->nrOrg, $this->dtCompetencia*/);
        if($cont == 0) {        //Verifica se houve registros a retroceder
            $this->msgErro .= "Não havia nenhum registro no magnético a retroceder da Organização ".$this->nrOrg." na competência ".$this->dtCompetencia.Dirf::ql_msg;
            return false;       //Retorna 'erro' (somente para instruir o facade a mostrar a mensagem de erro)
        }
        return $cont;           //Retorna a quantidade de registros retrocedidos
    }

    public function getQdeRegsMagnet(){    //Retorna a quantidade de registros no magnético
        if($this->nrOrg == -1) //Verifica se o formulário foi carregado ou se não houve outras falhas / inconsistências
            return false;
        if($this->idGeraMagnetic == 'S')
            return $this->objMagnetico->qdeRegistros();
        return 0;
    }

    public function getMensagens(){
        return $this->msgErro;
    }
} //FIM DA CLASSE DIRF
